import 'dart:convert';
import 'dart:typed_data';
import 'package:crypto_keys/crypto_keys.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_datetime_picker/flutter_datetime_picker.dart';
import 'package:http/http.dart' as http;
// import 'package:shared_preferences/shared_preferences.dart';
// import '../beevoter_common.dart';
import '../beevoter_crypto.dart';
import 'model.dart';

class VoteToBeCast {
  PoolModel pool;
  int voterPoolId;
  List<PoolOptionModel> votes;

  VoteToBeCast(this.pool, this.voterPoolId, this.votes);
}

class BeeVoterController {
  static String currentLanguage = 'pt-BR';
  static String translate(String value) { return value; }
  static BuildContext lastContext;
  static LocaleType localeType;
  static BeeVoterController lastController;
  // static SharedPreferences prefs;
  static bool keepMeLoggedIn = false;
  static GlobalResponse global;
  static refreshPref() async {
    // var os = new BeeVoterStoredData(
    //     usr: keepMeLoggedIn ? (lastController == null ? null : lastController.currentUser) : null,
    //     global: global,
    //     currentLanguage: currentLanguage,
    //     currentFont: DefaultTheme.currentFont,
    //     currentTheme: DefaultTheme.currentTheme,
    //     sessionId: (lastController == null ? null : (lastController.currentUser == null ? null : lastController.currentUser.sessionId)));
    // var js = os.toJson();
    // var jsTxt = jsonEncode(js);
    // if (prefs == null) prefs = await SharedPreferences.getInstance();
    // await prefs.setString('data', jsTxt);
  }
  static Future<Uint8List> get(String url) async {
    var response = await http.get('$lastController.apiHome$url');
    return response.bodyBytes;
  }
  String apiHome = "";

  BeeVoterController(String apiHome) {
    this.apiHome = apiHome+'/api/App/';
    this.challenges = [];
    this.currentUser = null;
    BeeVoterController.lastController = this;
  }

  InitializeResponse initializeResponse;
  Future startConnection() async {
    this.initializeResponse = await this.initialize(new InitializeRequest());
  }

  /*
  static Future<GetNewChallengeResponseModel> getNewChallenge(
      GetNewChallengeRequestModel model) async {
    return GetNewChallengeResponseModel.create(
        await post("GetNewChallenge", model.toJson()));
  }
  */

  Future<PublicAccessRequestModel> unauthCommand(PublicAccessRequestModel command) async {
    if (initializeResponse != null && initializeResponse.organization != null) {
      command.organizationId = initializeResponse.organization.id;
    }
    return command;
  }
  Future<PrivateAccessRequestModel> authCommand(PrivateAccessRequestModel command) async {
    if (initializeResponse != null && initializeResponse.organization != null && this.currentUser != null) {
      command.organizationId = initializeResponse.organization.id;
      command.sessionId = this.currentUser.sessionId;
      var challenge = await getChallengeFromCache();
      command.challengeId = challenge.id;
      command.response = CommonCrypto.calculateHmac(challenge.value, this.currentUser.passwordHash);
    }
    //
    // command.authenticationId = currentUser.authenticationId;
    // command.challenge = challenge;
    // command.language = currentLanguage;
    // command.response = currentUser.sign(challenge);
    // command.loginId = currentUser.loginId;
    // command.controllerSession = controllerSession;
    return command;
  }

  Future<dynamic> post(String action, Map<String, dynamic> js,
      {int timeout: 600}) async {
    var response = await http
        .post(apiHome + action,
        headers: {
          "Content-Type": "application/json",
          'Accept': 'application/json',
        },
        body: jsonEncode(js))
        .timeout(new Duration(seconds: timeout));
    if (response.statusCode != 200) {
      if (response.body.toString().indexOf('Challenge was responded') >= 0) {
       var rC = await getChallenge(new GetChallengeRequestModel());
       addChallenge(rC.challengeId, rC.challenge);
       return await post(action, js);
      }
      throw (response.reasonPhrase + ' - ' + response.body.toString());
    }
    // return null;
    return jsonDecode(response.body);
  }

  List<ChallengeModel> challenges;
  AuthenticatedUserModel currentUser;
  void addChallenge(int id, String challenge) {
    var model = new ChallengeModel(id: id, value: challenge);
    challenges.add(model);
  }
  Future<ChallengeModel> getChallengeFromCache() async {
    ChallengeModel ret;
    if (challenges.length > 0) {
      ret = challenges[challenges.length - 1];
      challenges.removeAt(challenges.length - 1);
    }
    if (ret == null) {
      var response = await getChallenge(new GetChallengeRequestModel());
      return new ChallengeModel(id: response.challengeId, value: response.challenge);
    }
    return ret;
  }
  Future login(String contact, CredentialType contactType, String password) async {
    var model = new StartPasswordSessionRequestModel(contact: contact, contactType: contactType, passwordHash: CommonCrypto.calculateSha256(password), passwordHashUpperCase: CommonCrypto.calculateSha256(password.toUpperCase()));
    try {
      StartPasswordSessionResponseModel response = await this
          .startPasswordSession(model);

      String pkcs12 = '';
      String publicKey = '';
      if (response.credential != null && response.credential.privateKey != null) {
      }
      this.currentUser = new AuthenticatedUserModel(
          adminPermissions: response.adminPermissions,
          sessionId: response.sessionId,
          passwordHash: response.isUpperCase ? model.passwordHashUpperCase : model.passwordHash,
          credential: response.credential,
          pkcs12: pkcs12,
          publicKey: publicKey);

    } catch (err) {
      print(err);
    }
  }

  Future<CastVotesRequestModel> createVoteModel(int voterId, KeyPairInfoModel voterKeyPair, String password, List<VoteToBeCast> votesToBeCast) async {
    var organizationKeyPair = CommonCrypto.getPublicKeyByKeyPair(this.initializeResponse.organization.privateKey);
    var votes = new CastVotesRequestModel(voterId: voterId, votes: []);
    for(var i=0; i<votesToBeCast.length; i++)
      votes.votes.add(await createPoolVote(organizationKeyPair, voterKeyPair, password, votesToBeCast[i].pool, votesToBeCast[i].voterPoolId, votesToBeCast[i].votes));
    return votes;
  }
  Future<VoteModel> createPoolVote(KeyPair organizationKeyPair, KeyPairInfoModel voterKeyPair, String password, PoolModel pool, int voterPoolId, List<PoolOptionModel> voteOptions) async {
    var openVotes = '';
    for(var i=0; i<voteOptions.length; i++) {
      if (i>0)
        openVotes += ';';
      openVotes += voteOptions[i].id.toString();
    }
    var encryptedVote = CommonCrypto.rsaEncrypt(organizationKeyPair, new Uint8List.fromList(openVotes.codeUnits));
    var voteSignature = await CommonCrypto.signFromPkcs12(password, voterKeyPair.pinSalt, voterKeyPair.iterationCount, voterKeyPair.pkcs12, encryptedVote.data);
    var reqString = CommonCrypto.getRandomString(10);
    var req = CommonCrypto.calculateSha256(reqString);
    return new VoteModel(
      voterId: voterPoolId,
      poolId: pool.id,
      encryptedVote: base64Encode(encryptedVote.data),
      openVote: pool.isSecret ? '' : openVotes,
      signature: base64Encode(voteSignature),
      receiptRequest: req,
      voterKeyId: voterKeyPair.id,
    );
  }

  Future<ActiveUsersResponseModel> activeUsers(ActiveUsersRequestModel model) async {
    return ActiveUsersResponseModel.create(await post("ActiveUsers", (await authCommand(model)).toJson()));
  }
  Future<AddContactByPublicResponseModel> addContactByPublic(AddContactByPublicRequestModel model) async {
    return AddContactByPublicResponseModel.create(await post("AddContactByPublic", (await unauthCommand(model)).toJson()));
  }
  Future<AddCredentialResponseModel> addCredential(AddCredentialRequestModel model) async {
    return AddCredentialResponseModel.create(await post("AddCredential", (await authCommand(model)).toJson()));
  }
  Future<AddIpToBlackListResponseModel> addIpToBlackList(AddIpToBlackListRequestModel model) async {
    return AddIpToBlackListResponseModel.create(await post("AddIpToBlackList", (await authCommand(model)).toJson()));
  }
  Future<AddIpToWhiteListResponseModel> addIpToWhiteList(AddIpToWhiteListRequestModel model) async {
    return AddIpToWhiteListResponseModel.create(await post("AddIpToWhiteList", (await authCommand(model)).toJson()));
  }
  Future<AddVoterResponseModel> addVoter(AddVoterRequestModel model) async {
    return AddVoterResponseModel.create(await post("AddVoter", (await authCommand(model)).toJson()));
  }
  Future<AdminEventsResponseModel> adminEvents(AdminEventsRequestModel model) async {
    return AdminEventsResponseModel.create(await post("AdminEvents", (await authCommand(model)).toJson()));
  }
  Future<AnswerPositiveIdentificationChallengeResponseModel> answerPositiveIdentificationChallenge(AnswerPositiveIdentificationChallengeRequestModel model) async {
    return AnswerPositiveIdentificationChallengeResponseModel.create(await post("AnswerPositiveIdentificationChallenge", (await unauthCommand(model)).toJson()));
  }
  Future<CastVotesResponseModel> castVotes(CastVotesRequestModel model) async {
    return CastVotesResponseModel.create(await post("CastVotes", (await authCommand(model)).toJson()));
  }
  Future<ChangePasswordResponseModel> changePassword(ChangePasswordRequestModel model) async {
    return ChangePasswordResponseModel.create(await post("ChangePassword", (await authCommand(model)).toJson()));
  }
  Future<ChatAddReasonMessageResponseModel> chatAddReasonMessage(ChatAddReasonMessageRequestModel model) async {
    return ChatAddReasonMessageResponseModel.create(await post("ChatAddReasonMessage", (await unauthCommand(model)).toJson()));
  }
  Future<ChatAssociateVoterResponseModel> chatAssociateVoter(ChatAssociateVoterRequestModel model) async {
    return ChatAssociateVoterResponseModel.create(await post("ChatAssociateVoter", (await authCommand(model)).toJson()));
  }
  Future<ChatCreateAttendantResponseModel> chatCreateAttendant(ChatCreateAttendantRequestModel model) async {
    return ChatCreateAttendantResponseModel.create(await post("ChatCreateAttendant", (await authCommand(model)).toJson()));
  }
  Future<ChatFinishConversationResponseModel> chatFinishConversation(ChatFinishConversationRequestModel model) async {
    return ChatFinishConversationResponseModel.create(await post("ChatFinishConversation", (await unauthCommand(model)).toJson()));
  }
  Future<ChatReadConversationResponseModel> chatReadConversation(ChatReadConversationRequestModel model) async {
    return ChatReadConversationResponseModel.create(await post("ChatReadConversation", (await unauthCommand(model)).toJson()));
  }
  Future<ChatReadConversationsResponseModel> chatReadConversations(ChatReadConversationsRequestModel model) async {
    return ChatReadConversationsResponseModel.create(await post("ChatReadConversations", (await authCommand(model)).toJson()));
  }
  Future<ChatReadMessageResponseModel> chatReadMessage(ChatReadMessageRequestModel model) async {
    return ChatReadMessageResponseModel.create(await post("ChatReadMessage", (await unauthCommand(model)).toJson()));
  }
  Future<ChatRequestSupportResponseModel> chatRequestSupport(ChatRequestSupportRequestModel model) async {
    return ChatRequestSupportResponseModel.create(await post("ChatRequestSupport", (await unauthCommand(model)).toJson()));
  }
  Future<ChatSendMessageToAttendantResponseModel> chatSendMessageToAttendant(ChatSendMessageToAttendantRequestModel model) async {
    return ChatSendMessageToAttendantResponseModel.create(await post("ChatSendMessageToAttendant", (await unauthCommand(model)).toJson()));
  }
  Future<ChatSendMessageToUserResponseModel> chatSendMessageToUser(ChatSendMessageToUserRequestModel model) async {
    return ChatSendMessageToUserResponseModel.create(await post("ChatSendMessageToUser", (await unauthCommand(model)).toJson()));
  }
  Future<EndSessionResponseModel> endSession(EndSessionRequestModel model) async {
    return EndSessionResponseModel.create(await post("EndSession", (await authCommand(model)).toJson()));
  }
  Future<EventMonitorItemResponseModel> eventMonitor(EventMonitorItemRequestModel model) async {
    return EventMonitorItemResponseModel.create(await post("EventMonitor", (await unauthCommand(model)).toJson()));
  }
  Future<ForcePoolStartResponseModel> forcePoolStart(ForcePoolStartRequestModel model) async {
    return ForcePoolStartResponseModel.create(await post("ForcePoolStart", (await authCommand(model)).toJson()));
  }
  Future<GenerateFaceToFaceBallotBoxCodeResponseModel> generateFaceToFaceBallotBoxCode(GenerateFaceToFaceBallotBoxCodeRequestModel model) async {
    return GenerateFaceToFaceBallotBoxCodeResponseModel.create(await post("GenerateFaceToFaceBallotBoxCode", (await authCommand(model)).toJson()));
  }
  Future<GetAdminEventResponseModel> getAdminEvent(GetAdminEventRequestModel model) async {
    return GetAdminEventResponseModel.create(await post("GetAdminEvent", (await authCommand(model)).toJson()));
  }
  Future<GetChallengeResponseModel> getChallenge(GetChallengeRequestModel model) async {
    return GetChallengeResponseModel.create(await post("GetChallenge", (await unauthCommand(model)).toJson()));
  }
  Future<GetIPEventResponseModel> getIPEvent(GetIPEventRequestModel model) async {
    return GetIPEventResponseModel.create(await post("GetIPEvent", (await authCommand(model)).toJson()));
  }
  Future<GetPasswordForCertificateResponseModel> getPasswordForCertificate(GetPasswordForCertificateRequestModel model) async {
    return GetPasswordForCertificateResponseModel.create(await post("GetPasswordForCertificate", (await unauthCommand(model)).toJson()));
  }
  Future<GetPositiveIdentificationChallengeResponseModel> getPositiveIdentificationChallenge(GetPositiveIdentificationChallengeRequestModel model) async {
    return GetPositiveIdentificationChallengeResponseModel.create(await post("GetPositiveIdentificationChallenge", (await unauthCommand(model)).toJson()));
  }
  Future<GetUserAgentEventResponseModel> getUserAgentEvent(GetUserAgentEventRequestModel model) async {
    return GetUserAgentEventResponseModel.create(await post("GetUserAgentEvent", (await authCommand(model)).toJson()));
  }
  Future<GetUserContactsResponseModel> getUserContacts(GetUserContactsRequestModel model) async {
    return GetUserContactsResponseModel.create(await post("GetUserContacts", (await unauthCommand(model)).toJson()));
  }
  Future<ImportPositiveIdentificationNamesResponseModel> importPositiveIdentificationNames(ImportPositiveIdentificationNamesRequestModel model) async {
    return ImportPositiveIdentificationNamesResponseModel.create(await post("ImportPositiveIdentificationNames", (await authCommand(model)).toJson()));
  }
  Future<ImportVotersResponseModel> importVoters(ImportVotersRequestModel model) async {
    return ImportVotersResponseModel.create(await post("ImportVoters", (await authCommand(model)).toJson()));
  }
  Future<InitializeResponse> initialize(InitializeRequest model) async {
    return InitializeResponse.create(await post("Initialize", model.toJson()));
  }
  Future<InvalidateVoteResponseModel> invalidateVote(InvalidateVoteRequestModel model) async {
    return InvalidateVoteResponseModel.create(await post("InvalidateVote", (await authCommand(model)).toJson()));
  }
  Future<ListManagedIpsResponseModel> listManagedIps(ListManagedIpsRequestModel model) async {
    return ListManagedIpsResponseModel.create(await post("ListManagedIps", (await authCommand(model)).toJson()));
  }
  Future<ListRegisteredPINsResponseModel> listRegisteredPINs(ListRegisteredPINsRequestModel model) async {
    return ListRegisteredPINsResponseModel.create(await post("ListRegisteredPINs", (await authCommand(model)).toJson()));
  }
  Future<ListSharedGroupsResponseModel> listSharedGroups(ListSharedGroupsRequestModel model) async {
    return ListSharedGroupsResponseModel.create(await post("ListSharedGroups", (await authCommand(model)).toJson()));
  }
  Future<LoginFaceToFaceBallotBoxResponseModel> loginFaceToFaceBallotBox(LoginFaceToFaceBallotBoxRequestModel model) async {
    return LoginFaceToFaceBallotBoxResponseModel.create(await post("LoginFaceToFaceBallotBox", (await authCommand(model)).toJson()));
  }
  Future<PoolsStatusResponseModel> poolsStatus(PoolsStatusRequestModel model) async {
    return PoolsStatusResponseModel.create(await post("PoolsStatus", (await authCommand(model)).toJson()));
  }
  Future<ReadFaceToFaceBallotBoxesResponseModel> readFaceToFaceBallotBoxes(ReadFaceToFaceBallotBoxesRequestModel model) async {
    return ReadFaceToFaceBallotBoxesResponseModel.create(await post("ReadFaceToFaceBallotBoxes", (await authCommand(model)).toJson()));
  }
  Future<ReceiptVerifyResponseModel> receiptVerify(ReceiptVerifyRequestModel model) async {
    return ReceiptVerifyResponseModel.create(await post("ReceiptVerify", (await unauthCommand(model)).toJson()));
  }
  Future<RegisterPINResponseModel> registerPIN(RegisterPINRequestModel model) async {
    return RegisterPINResponseModel.create(await post("RegisterPIN", (await authCommand(model)).toJson()));
  }
  Future<ReleaseIPResponseModel> releaseIP(ReleaseIPRequestModel model) async {
    return ReleaseIPResponseModel.create(await post("ReleaseIP", (await authCommand(model)).toJson()));
  }
  Future<RemoveIpFromBlackListResponseModel> removeIpFromBlackList(RemoveIpFromBlackListRequestModel model) async {
    return RemoveIpFromBlackListResponseModel.create(await post("RemoveIpFromBlackList", (await authCommand(model)).toJson()));
  }
  Future<RemoveIpFromWhiteListResponseModel> removeIpFromWhiteList(RemoveIpFromWhiteListRequestModel model) async {
    return RemoveIpFromWhiteListResponseModel.create(await post("RemoveIpFromWhiteList", (await authCommand(model)).toJson()));
  }
  Future<RenderTemplateUploadResponseModel> renderTemplateUpload(RenderTemplateUploadRequestModel model) async {
    return RenderTemplateUploadResponseModel.create(await post("RenderTemplateUpload", (await authCommand(model)).toJson()));
  }
  Future<ResetPasswordFinishResponseModel> resetPasswordFinish(ResetPasswordFinishRequestModel model) async {
    return ResetPasswordFinishResponseModel.create(await post("ResetPasswordFinish", (await unauthCommand(model)).toJson()));
  }
  Future<ResetPasswordStartResponseModel> resetPasswordStart(ResetPasswordStartRequestModel model) async {
    return ResetPasswordStartResponseModel.create(await post("ResetPasswordStart", (await unauthCommand(model)).toJson()));
  }
  Future<SaveConfigurationResponseModel> saveConfiguration(SaveConfigurationRequestModel model) async {
    return SaveConfigurationResponseModel.create(await post("SaveConfiguration", (await authCommand(model)).toJson()));
  }
  Future<SearchVoterResponseModel> searchVoter(SearchVoterRequestModel model) async {
    return SearchVoterResponseModel.create(await post("SearchVoter", (await authCommand(model)).toJson()));
  }
  Future<SearchVoterByDocumentResponseModel> searchVoterByDocument(SearchVoterByDocumentRequestModel model) async {
    return SearchVoterByDocumentResponseModel.create(await post("SearchVoterByDocument", (await unauthCommand(model)).toJson()));
  }
  Future<SearchVotersResponseModel> searchVoters(SearchVotersRequestModel model) async {
    return SearchVotersResponseModel.create(await post("SearchVoters", (await authCommand(model)).toJson()));
  }
  Future<SendTokenByCredentialResponseModel> sendTokenByCredential(SendTokenByCredentialRequestModel model) async {
    return SendTokenByCredentialResponseModel.create(await post("SendTokenByCredential", (await authCommand(model)).toJson()));
  }
  Future<SendVoterFaceToFaceBallotBoxResponseModel> sendVoterFaceToFaceBallotBox(SendVoterFaceToFaceBallotBoxRequestModel model) async {
    return SendVoterFaceToFaceBallotBoxResponseModel.create(await post("SendVoterFaceToFaceBallotBox", (await authCommand(model)).toJson()));
  }
  Future<StartPasswordSessionResponseModel> startPasswordSession(StartPasswordSessionRequestModel model) async {
    return StartPasswordSessionResponseModel.create(await post("StartPasswordSession", (await unauthCommand(model)).toJson()));
  }
  Future<TokenByLinkResponseModel> tokenByLink(TokenByLinkRequestModel model) async {
    return TokenByLinkResponseModel.create(await post("TokenByLink", (await unauthCommand(model)).toJson()));
  }
  Future<ToogleCredentialResponseModel> toogleCredential(ToogleCredentialRequestModel model) async {
    return ToogleCredentialResponseModel.create(await post("ToogleCredential", (await authCommand(model)).toJson()));
  }
  Future<TooglePoolResponseModel> tooglePool(TooglePoolRequestModel model) async {
    return TooglePoolResponseModel.create(await post("TooglePool", (await authCommand(model)).toJson()));
  }
  Future<ToogleVoterPoolResponseModel> toogleVoterPool(ToogleVoterPoolRequestModel model) async {
    return ToogleVoterPoolResponseModel.create(await post("ToogleVoterPool", (await authCommand(model)).toJson()));
  }
  Future<UserAgentMonitorItemResponseModel> userAgentMonitor(UserAgentMonitorItemRequestModel model) async {
    return UserAgentMonitorItemResponseModel.create(await post("UserAgentMonitor", (await unauthCommand(model)).toJson()));
  }
  Future<VoteMonitorItemResponseModel> voteMonitor(VoteMonitorItemRequestModel model) async {
    return VoteMonitorItemResponseModel.create(await post("VoteMonitor", (await unauthCommand(model)).toJson()));
  }
}

class AuthenticatedUserModel {
  AuthenticatedUserModel({this.adminPermissions, this.sessionId, this.passwordHash, this.credential, this.pkcs12, this.publicKey});

  AdministratorPermission adminPermissions;
  int sessionId;
  String passwordHash;
  CredentialModel credential;
  String pkcs12;
  String publicKey;

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> json = new Map<String, dynamic>();
    json['adminPermissions'] = this.adminPermissions.value;
    json['sessionId'] = this.sessionId;
    json['passwordHash'] = this.passwordHash;
    if (this.credential != null) json['credential'] = this.credential.toJson();
    json['pkcs12'] = this.pkcs12;
    json['publicKey'] = this.publicKey;
    return json;
  }

  fromJson(Map<String, dynamic> json) {
    this.adminPermissions.value = json['adminPermissions'];
    this.sessionId = json['sessionId'];
    this.passwordHash = json['passwordHash'];
    if (json['credential'] != null) this.credential = CredentialModel.create(json['credential']);
    this.pkcs12 = json['pkcs12'];
    this.publicKey = json['publicKey'];
  }

  static AuthenticatedUserModel create(Map<String, dynamic> json) {
    var ret = new AuthenticatedUserModel();
    ret.fromJson(json);
    return ret;
  }

}
class BeeVoterStoredData {
  AuthenticatedUserModel usr;
  GlobalResponse global;
  String currentLanguage;
  String currentTheme;
  String currentFont;
  int sessionId;

  BeeVoterStoredData(
      {
        this.usr,
        this.global,
        this.currentLanguage,
        this.currentTheme,
        this.currentFont,
        this.sessionId});

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> json = new Map<String, dynamic>();
    if (this.usr != null) json['usr'] = this.usr.toJson();
    if (this.global != null) json['global'] = this.global.toJson();
    json['currentLanguage'] = this.currentLanguage;
    json['currentTheme'] = this.currentTheme;
    json['currentFont'] = this.currentFont;
    if (this.sessionId != null) json['sessionId'] = this.sessionId;
    return json;
  }

  fromJson(Map<String, dynamic> json) {
    if (json['usr'] != null)
      this.usr = AuthenticatedUserModel.create(json['usr']);
    if (json['global'] != null)
      this.global = GlobalResponse.create(json['global']);
    if (json['currentLanguage'] != null)
      this.currentLanguage = json['currentLanguage'].toString();
    if (json['currentTheme'] != null)
      this.currentTheme = json['currentTheme'].toString();
    if (json['currentFont'] != null)
      this.currentFont = json['currentFont'].toString();
    if (json['sessionId'] != null) this.sessionId = json['sessionId'];
  }

  static BeeVoterStoredData create(Map<String, dynamic> json) {
    var ret = new BeeVoterStoredData();
    ret.fromJson(json);
    return ret;
  }
}
class GlobalResponse {
  GlobalResponse(); // { }
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> json = new Map<String, dynamic>();
    // json['monthlyBasicUserValue'] = this.monthlyBasicUserValue;
    return json;
  }

  fromJson(Map<String, dynamic> json) {
    // this.monthlyBasicUserValue = json['monthlyBasicUserValue'];
  }
  static GlobalResponse create(Map<String, dynamic> json) {
    var ret = new GlobalResponse();
    ret.fromJson(json);
    return ret;
  }
}
